#include "f.hpp"

#include "dco.hpp"

#include "Eigen/Dense"

#include<iostream>
#include<cassert>

template<typename T, int N>
void f_a(const Eigen::Matrix<T,N,1>& x_v, T& y_v, Eigen::Matrix<T,N,1>& dydx) {
  using DCO_M=typename dco::ga1s<T>;
  using DCO_T=typename DCO_M::type;
  auto n=x_v.size();
  Eigen::Matrix<DCO_T,N,1> x(n); DCO_T y=0;
  DCO_M::global_tape=DCO_M::tape_t::create();
  for (auto i=0;i<n;i++) {
    x(i)=x_v(i);
    DCO_M::global_tape->register_variable(x(i));
  }
  f(x,y);
  y_v=dco::value(y);
  dco::derivative(y)=1;
  DCO_M::global_tape->interpret_adjoint();
  for (auto i=0;i<n;i++) dydx(i)=dco::derivative(x(i));
  DCO_M::tape_t::remove(DCO_M::global_tape);
}

int main(int argc, char* argv[]) {
  assert(argc==2); auto n=std::stoi(argv[1]);
  using T=double;
  using VT=Eigen::Matrix<T,Eigen::Dynamic,1>; 
  VT x=VT::Random(n), dydx(n); 
  T y;
  f_a(x,y,dydx);
  std::cout << y << std::endl << dydx << std::endl;
  return 0;
}
