#include "f.hpp"

#include "Eigen/Dense"

#include<iostream>
#include <limits>
#include<cassert>

template<typename T, int N>
void f_cfd(Eigen::Matrix<T,N,1> x, T& y, Eigen::Matrix<T,N,1>& dydx) {
  f(x,y);
  for (auto i=0;i<x.size();i++) {
    T dx=fabs(x(i))<1 ? sqrt(std::numeric_limits<T>::epsilon())
       : sqrt(std::numeric_limits<T>::epsilon())*fabs(x(i));
    T yp,ym;
    x(i)+=dx; f(x,yp); x(i)-=2*dx; f(x,ym); x(i)+=dx;
    dydx(i)=(yp-ym)/(2*dx);
  }
}

int main(int argc, char* argv[]) {
  assert(argc==2); auto n=std::stoi(argv[1]);
  using T=double; 
  using VT=Eigen::Matrix<T,Eigen::Dynamic,1>; 
  VT x=VT::Random(n), dydx(n); 
  T y;
  f_cfd(x,y,dydx);
  std::cout << y << std::endl
            << dydx << std::endl;
  return 0;
}
