#include "f.hpp"
#include "dco.hpp"
#include "Eigen/Dense"

#include<iostream>
#include<cassert>

template<typename T, int N, int M>
void f_a(const Eigen::Matrix<T,N,1>& x_v, Eigen::Matrix<T,M,1>& y_v, Eigen::Matrix<T,M,N>& dydx) {
  using DCO_M=typename dco::ga1s<T>; using DCO_T=typename DCO_M::type;
  auto n=x_v.size(), m=y_v.size();
  Eigen::Matrix<DCO_T,N,1> x(n); Eigen::Matrix<DCO_T,M,1> y(m); 
  for (auto j=0;j<m;j++) y(j)=y_v(j);
  DCO_M::global_tape=DCO_M::tape_t::create();
  for (auto i=0;i<n;i++) {
    x(i)=x_v(i);
    DCO_M::global_tape->register_variable(x(i));
  }
  f(x,y);
  for (auto j=0;j<m;j++) y_v(j)=dco::value(y(j));
  for (auto j=0;j<m;j++) {
    dco::derivative(y(j))=1;
    DCO_M::global_tape->interpret_adjoint();
    for (auto i=0;i<n;i++) dydx(j,i)=dco::derivative(x(i));
    DCO_M::global_tape->zero_adjoints();
  }  
  DCO_M::tape_t::remove(DCO_M::global_tape);
}

int main(int argc, char* argv[]) {
  assert(argc==3);
  auto n=std::stoi(argv[1]), m=std::stoi(argv[2]);
  using VT=Eigen::Matrix<double,Eigen::Dynamic,1>; 
  using MT=Eigen::Matrix<double,Eigen::Dynamic,Eigen::Dynamic>; 
  VT x=VT::Random(n), y=VT::Random(m); 
  MT dydx(m,n); 
  f_a(x,y,dydx);
  std::cout << y << std::endl << dydx << std::endl;
  return 0;
}
