#include "f.hpp"
#include "Eigen/Dense"

#include<iostream>
#include <limits>
#include<cassert>

template<typename T, int N, int M>
void f_cfd(Eigen::Matrix<T,N,1>& x, Eigen::Matrix<T,M,1>& y, Eigen::Matrix<T,M,N>& dydx) {
  for (auto i=0;i<x.size();i++) {
    T dx=fabs(x(i))<1 ? sqrt(std::numeric_limits<T>::epsilon())
       : sqrt(std::numeric_limits<T>::epsilon())*fabs(x(i));
    Eigen::Matrix<T,M,1> yp(y),ym(y);
    x(i)+=dx; f(x,yp); x(i)-=2*dx; f(x,ym); x(i)+=dx;
    for (auto j=0;j<y.size();j++) dydx(j,i)=(yp(j)-ym(j))/(2*dx);
  }
  f(x,y);
}

int main(int argc, char* argv[]) {
  assert(argc==3);
  auto n=std::stoi(argv[1]);
  auto m=std::stoi(argv[2]);
  using VT=Eigen::Matrix<double,Eigen::Dynamic,1>; 
  using MT=Eigen::Matrix<double,Eigen::Dynamic,Eigen::Dynamic>; 
  VT x=VT::Random(n), y=VT::Random(m);
  MT dydx(m,n); 
  f_cfd(x,y,dydx);
  std::cout << y << std::endl
            << dydx << std::endl;
  return 0;
}
